/* dia-canvas.h
 * Copyright (C) 2000  James Henstridge, Arjan Molenaar
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef __DIA_CANVAS_H__
#define __DIA_CANVAS_H__

#include <glib-object.h>
#include <pango/pango-layout.h>
#include <diacanvas/dia-event.h>
#include <diacanvas/dia-solver.h>
#include <diacanvas/dia-geometry.h>
#include <diacanvas/dia-shape.h>

/*
// losing this lets us test speed
#undef g_message
static void pruntf(char const *igMe, ...) {}
#define g_message if(0);else pruntf // nada
*/

G_BEGIN_DECLS

#define DIA_TYPE_CANVAS			(dia_canvas_get_type ())
#define DIA_CANVAS(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), DIA_TYPE_CANVAS, DiaCanvas))
#define DIA_CANVAS_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST ((klass), DIA_TYPE_CANVAS, DiaCanvasClass))
#define DIA_IS_CANVAS(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), DIA_TYPE_CANVAS))
#define DIA_IS_CANVAS_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), DIA_TYPE_CANVAS))
#define DIA_CANVAS_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), DIA_TYPE_CANVAS, DiaCanvasClass))

#define DIA_TYPE_CANVAS_ITEM		(dia_canvas_item_get_type ())
#define DIA_CANVAS_ITEM(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), DIA_TYPE_CANVAS_ITEM, DiaCanvasItem))
#define DIA_CANVAS_ITEM_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), DIA_TYPE_CANVAS_ITEM, DiaCanvasItemClass))
#define DIA_IS_CANVAS_ITEM(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), DIA_TYPE_CANVAS_ITEM))
#define DIA_IS_CANVAS_ITEM_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), DIA_TYPE_CANVAS_ITEM))
#define DIA_CANVAS_ITEM_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), DIA_TYPE_CANVAS_ITEM, DiaCanvasItemClass))

#define DIA_TYPE_CANVAS_GROUPABLE	(dia_canvas_groupable_get_type ())
#define DIA_CANVAS_GROUPABLE(obj)	(G_TYPE_CHECK_INSTANCE_CAST ((obj), DIA_TYPE_CANVAS_GROUPABLE, DiaCanvasItem))
#define DIA_IS_CANVAS_GROUPABLE(obj)	(G_TYPE_CHECK_INSTANCE_TYPE ((obj), DIA_TYPE_CANVAS_GROUPABLE))
#define DIA_CANVAS_GROUPABLE_GET_IFACE(obj)  (G_TYPE_INSTANCE_GET_INTERFACE ((obj), DIA_TYPE_CANVAS_GROUPABLE, DiaCanvasGroupable))

#define DIA_TYPE_CANVAS_GROUP		(dia_canvas_group_get_type ())
#define DIA_CANVAS_GROUP(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), DIA_TYPE_CANVAS_GROUP, DiaCanvasGroup))
#define DIA_CANVAS_GROUP_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), DIA_TYPE_CANVAS_GROUP, DiaCanvasGroupClass))
#define DIA_IS_CANVAS_GROUP(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), DIA_TYPE_CANVAS_GROUP))
#define DIA_IS_CANVAS_GROUP_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), DIA_TYPE_CANVAS_GROUP))
#define DIA_CANVAS_GROUP_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), DIA_TYPE_CANVAS_GROUP, DiaCanvasGroupClass))

/* Type for DiaCanvasItem::affine. */
#define DIA_TYPE_AFFINE dia_canvas_item_affine_get_type ()
GType dia_canvas_item_affine_get_type (void);

/* Type for DiaCanvasIter */
#define DIA_TYPE_CANVAS_ITER dia_canvas_iter_get_type ()
GType dia_canvas_iter_get_type (void);

/* Type for DiaCanvasItem::handles. */
#define DIA_TYPE_CANVAS_ITEM_HANDLES dia_canvas_item_handles_get_type ()
GType dia_canvas_item_handles_get_type (void);

typedef struct _DiaCanvas		DiaCanvas;
typedef struct _DiaCanvasClass		DiaCanvasClass;

typedef struct _DiaCanvasItem		DiaCanvasItem; 
typedef struct _DiaCanvasItemClass	DiaCanvasItemClass;

typedef struct _DiaCanvasIter		DiaCanvasIter;
typedef struct _DiaCanvasGroupable	DiaCanvasGroupable;

typedef struct _DiaCanvasGroup		DiaCanvasGroup;
typedef struct _DiaCanvasGroupClass	DiaCanvasGroupClass;

/* Object is declared in dia-handle.h. */
typedef struct _DiaHandle		DiaHandle;
typedef struct _DiaHandleClass		DiaHandleClass;

struct _DiaCanvasIter
{
	gpointer data[3];
	gint stamp;
};

#define DIA_SET_FLAGS(obj,flag)    G_STMT_START{ ((obj)->flags |= (flag)); }G_STMT_END
#define DIA_UNSET_FLAGS(obj,flag)  G_STMT_START{ ((obj)->flags &= ~(flag)); }G_STMT_END

#include <diacanvas/dia-handle.h>

/**
 * DiaCanvas
 *
 * DiaCanvas holds the data structure that represents the content that is
 * displayed in a #DiaCanvasView.
 * One DiaCanvas can have zero or more views.
 **/
struct _DiaCanvas
{
	GObject parent;
	
	/*< public, read only >*/
	gboolean static_extents;
	gboolean snap_to_grid;
	gboolean allow_undo;
	gboolean in_undo;

	DiaRectangle extents; 
	
	/* the root item in the canvas -- is a group */
	DiaCanvasItem *root;

	/* Variables for a grid: */
	gdouble interval_x;
	gdouble interval_y;
	gdouble offset_x;
	gdouble offset_y;
	DiaColor grid_color; /* RGB grid color */
	DiaColor grid_bg; /* RGB background color. */

	DiaSolver *solver;

	/*< private >*/
	guint idle_id;

	/* Undo/redo behavior */
	guint stack_depth;
	GSList *undo;
	GSList *redo;
	gpointer undo_entry;
};

struct _DiaCanvasClass
{
	GObjectClass parent_class;

	void (* update)		(DiaCanvas *canvas);

	/* Signals go here */

	/**
	 * extents_changed:
	 * @canvas:
	 * @new_extents:
	 *
	 * If the canvas' extents are changed, this function is called.
	 **/
	void (* extents_changed) (DiaCanvas *canvas, DiaRectangle *new_extents);

	/**
	 * redraw_view:
	 * @canvas:
	 *
	 * This signal should force all its views to redraw.
	 **/
	void (* redraw_view)	(DiaCanvas *canvas);

	/**
	 * undo:
	 * @canvas:
	 *
	 * This signal is emited every time a undo/redo action is done and
	 * every time undo information is added to the undo stack. User
	 * interfaces can connect to this signal and update undo/redo buttons,
	 * etc.
	 **/
	void (* undo)		(DiaCanvas *canvas);
};

GType	dia_canvas_get_type		(void);

DiaCanvas* dia_canvas_new		(void);

void	dia_canvas_request_update	(DiaCanvas *canvas);
void	dia_canvas_update_now		(DiaCanvas *canvas);

void	dia_canvas_resolve_now		(DiaCanvas *canvas);

void	dia_canvas_set_extents		(DiaCanvas *canvas,
					 const DiaRectangle *extents);
void	dia_canvas_set_static_extents	(DiaCanvas *canvas, gboolean stat);

void	dia_canvas_snap_to_grid		(DiaCanvas *canvas,
					 gdouble *x, gdouble *y);
void	dia_canvas_set_snap_to_grid	(DiaCanvas *canvas, gboolean snap);

gdouble	dia_canvas_glue_handle		(DiaCanvas *canvas,
					 const DiaHandle *handle,
					 const gdouble dest_x,
					 const gdouble dest_y,
					 gdouble *glue_x, gdouble *glue_y,
					 DiaCanvasItem **item);

GList*	dia_canvas_find_objects_in_rectangle (DiaCanvas *canvas,
					      DiaRectangle *rect);

void	dia_canvas_add_constraint	(DiaCanvas *canvas, DiaConstraint *c);
void	dia_canvas_remove_constraint	(DiaCanvas *canvas, DiaConstraint *c);

PangoLayout* dia_canvas_new_pango_layout (void);

void	dia_canvas_redraw_views		(DiaCanvas *canvas);

/* Undo/Redo */
/* Store a variable on the stack */
void	dia_canvas_preserve		(DiaCanvas *canvas, GObject *object,
					 const char *property_name,
					 const GValue *value, gboolean last);
void	dia_canvas_preserve_property	(DiaCanvas *canvas, GObject *object,
					 const char *property_name);
void	dia_canvas_preserve_property_last (DiaCanvas *canvas, GObject *object,
					   const char *property_name);

void	dia_canvas_push_undo		(DiaCanvas *canvas,
					 const char* optional_comment);
void	dia_canvas_pop_undo		(DiaCanvas *canvas);
void	dia_canvas_clear_undo		(DiaCanvas *canvas);
guint	dia_canvas_get_undo_depth	(DiaCanvas *canvas);

/* Redo last undo change: */
void	dia_canvas_pop_redo		(DiaCanvas *canvas);
void	dia_canvas_clear_redo		(DiaCanvas *canvas);
guint	dia_canvas_get_redo_depth	(DiaCanvas *canvas);

/* Also as property stack_depth */
void	dia_canvas_set_undo_stack_depth (DiaCanvas *canvas, guint depth);
guint	dia_canvas_get_undo_stack_depth (DiaCanvas *canvas);


/**
 * DiaCanvasItem
 *
 * DiaCanvasItem is the base class for all objects that are displayed in
 * a #DiaCanvasView. DiaCanvasItems are held by a #DiaCanvas.
 * Since a #DiaCanvas can have more than one view, every view holds some
 * custom information in a #DiaCanvasViewItem.
 * If a #DiaCanvasView is provided with a #DiaHandleLayer, the canvas is
 * able to view the handle and sent events to the handles.
 **/

/**
 * DiaCanvasItemFlags
 * @DIA_VISIBLE: If true, the item is visible. (default=TRUE)
 * @DIA_INTERACTIVE: If true, the item has interactive capabilities
 *  (it can handle events). (Default=TRUE).
 * @DIA_COMPOSITE: The item is an aggregate of it's parent. This means that
 *  if the item is selected or gets the focus, the parent will act and draw
 *  as if it was selected itself. (Default=FALSE)
 * @DIA_NEED_UPDATE: The item's update method should be executed. This flag
 *  is set by dia_canvas_item_request_update(). It should not be set in the
 *  constructor.
 * @DIA_UPDATE_ALL: Tell that all child objects (if any) have been requesting
 *  an update. This flag is used to optimize update requests.
 * @DIA_SELECT: The canvas is selected. This flag is set before the event
 *  handler is called. This flag is view specific. Note that a DiaCanvasItem
 *  is stateless, so you can not rely on this flag (nor DIA_FOCUS or DIA_GRAB)
 *  outside the event handler.
 * @DIA_FOCUS: The item has the focus. Same constraints as DIA_SELECT.
 * @DIA_GRAB: The item grabs all events. Same constraints as DIA_SELECT.
 * 
 * Flags that can be set on a #DiaCanvasItem.
 **/
typedef enum {
	DIA_VISIBLE	= 1,
	DIA_INTERACTIVE = 1 << 1,
	DIA_COMPOSITE	= 1 << 2,
	DIA_NEED_UPDATE	= 1 << 3,
	DIA_UPDATE_ALL	= 1 << 4,
	DIA_SELECT	= 1 << 8,
	DIA_FOCUS	= 1 << 9,
	DIA_GRAB	= 1 << 10
} DiaCanvasItemFlags;

#define DIA_CANVAS_ITEM_FLAGS(obj)	(DIA_CANVAS_ITEM (obj)->flags)

#define DIA_CANVAS_ITEM_INTERACTIVE(obj) ((DIA_CANVAS_ITEM_FLAGS (obj) & DIA_INTERACTIVE) != 0)
#define DIA_CANVAS_ITEM_COMPOSITE(obj) ((DIA_CANVAS_ITEM_FLAGS (obj) & DIA_COMPOSITE) != 0)
#define DIA_CANVAS_ITEM_VISIBLE(obj) ((DIA_CANVAS_ITEM_FLAGS (obj) & DIA_VISIBLE) != 0)
#define DIA_CANVAS_ITEM_NEED_UPDATE(obj) ((DIA_CANVAS_ITEM_FLAGS (obj) & DIA_NEED_UPDATE) != 0)
#define DIA_CANVAS_ITEM_UPDATE_ALL(obj) ((DIA_CANVAS_ITEM_FLAGS (obj) & DIA_UPDATE_ALL) != 0)
/* Only useful within an event handler: */
#define DIA_CANVAS_ITEM_FOCUS(obj) ((DIA_CANVAS_ITEM_FLAGS (obj) & DIA_FOCUS) != 0)
#define DIA_CANVAS_ITEM_GRAB(obj) ((DIA_CANVAS_ITEM_FLAGS (obj) & DIA_GRAB) != 0)
#define DIA_CANVAS_ITEM_SELECT(obj) ((DIA_CANVAS_ITEM_FLAGS (obj) & DIA_SELECT) != 0)

struct _DiaCanvasItem
{
	GObject		object;
	
	/*< public, read-only >*/
	guint32		flags;

	DiaCanvas      *canvas; /* Weak */
	DiaCanvasItem  *parent; /* Weak. Parent should support the
				 * DiaCanvasGroupable interface. */
	DiaRectangle    bounds;

	/* The items collection of handles. */
	GList *handles;

	/* Handles from other items that are connected. */
	GList *connected_handles;

	/* Affine transformation matrix (zoom, rotation, offset):
	 * x' = x*a[0] + y*a[2] + a[4]
	 * y' = x*a[1] + y*a[3] + a[5]
	 */
	gdouble affine[6];
};

struct _DiaCanvasItemClass
{
	GObjectClass parent_class;

	/**
	 * update: 
	 * @item: 
	 *
	 * Update the internal state of the canvas item. This includes:
	 * - recreating shapes
	 * - updating the bounding box
	 **/
	void	(* update)	(DiaCanvasItem *item, gdouble affine[6]);
	
	/**
	 * get_shape_iter:
	 * @item:
	 * @iter:
	 *
	 * Get a shape iterator.
	 **/
	gboolean (* get_shape_iter) (DiaCanvasItem *item, DiaCanvasIter *iter);

	/**
	 * next_shape:
	 * @item:
	 * @shape: previous shape
	 *
	 **/
	gboolean (* shape_next) (DiaCanvasItem *item, DiaCanvasIter *iter);

	/**
	 * shape_value:
	 * @item: 
	 * @iter: 
	 *
	 * Get the DiaShape that belongs to the iter
	 **/
	DiaShape* (* shape_value) (DiaCanvasItem *item, DiaCanvasIter *iter);

	/**
	 * point:
	 * @item:
	 * @x: Point, in item coordinates
	 * @y: 
	 *
	 * Return the distance from an item to a specific point.
	 * @x and @y are the coordinates relative to the item.
	 **/
	gdouble	(* point)	(DiaCanvasItem *item, gdouble x, gdouble y);
	
	/**
	 * handle_motion:
	 * @item:
	 * @handle: Handle to be moved.
	 * @wx: Point where the handle will be moved to, in world coordinates!
	 * @wy:
	 * @mask: modifiers pressed (like shift or control)
	 *
	 * If a handles moves, the #DiaCanvasItem is notified by this function.
	 * This function is called before #DiaHandle's attributes are updated.
	 * This function is used to adjust the item's movement where nessesary.
	 * @wx and @wy are in world coordinates.
	 **/
	void	(* handle_motion) (DiaCanvasItem *item, DiaHandle *handle,
				   gdouble *wx, gdouble *wy,
				   DiaEventMask mask);

	/**
	 * glue:
	 * @item:
	 * @handle:
	 * @x:
	 * @y:
	 *
	 * If a handle is near an object, that object is asked if it wants
	 * to connect the handle. Glueing is the first step: the item
	 * calculates the point where the handle should connect and the
	 * distance between the handles current position and the connection
	 * point is returned.
	 *
	 * Once the handle is released (the mouse button is released) the
	 * handle will request a connection to the item by calling the
	 * connect() function (see below).
	 *
	 * Returns: Distance from (@wx, @wy) to @item's closest connection
	 *	    point.
	 **/
	gdouble (* glue)	(DiaCanvasItem *item, DiaHandle *handle,
				 gdouble *wx, gdouble *wy);

	/*
	 * Signals
	 */

	/* Event is given in coordinates relative to the canvas item.
	 */
	gboolean (* event)	(DiaCanvasItem *item, DiaEvent *event);

	/* The item is moved. 
	 */
	void	(* move)	(DiaCanvasItem *item,
				 gdouble dx, gdouble dy, gboolean interactive);

	/* A handle may connect to another object.
	 * The position is calculated here. This function may move the handle
	 * in order to place it on a nice point for connection.
	 */
	gboolean (* connect)	(DiaCanvasItem *item, DiaHandle *handle);

	/* A handle can be disconnected if it is no longer attached to an
	 * object or if it is to be connected to another object. If the hanlde
	 * is just placed at some other place on the same object no disconnect()
	 * is send.
	 */
	gboolean (* disconnect)	(DiaCanvasItem *item, DiaHandle *handle);

	/*
	 * The following signals are used to keep the views up to date
	 * with the canvas model.
	 */

	/* Fire this one if an object needs updating.
	 */
	void	(* need_update)	(DiaCanvasItem *item);
	
	/* The z-order of the object changed by `positions' places.
	 * Positive values indicate raising to the top, negative values
	 * tell you the item is lowered.
	 */
	void	(* z_order)	(DiaCanvasItem *item, gint positions);

	/* The state of the object has changed. View items should act
	 * by setting the state of the object. Note that the state can only
	 * change when the item is handling an event through the event()
	 * callback. So in the end only the active view will be updated.
	 */
	void	(* state_changed) (DiaCanvasItem *item, gint new_state);
};


GType	dia_canvas_item_get_type	(void);

DiaCanvasItem* dia_canvas_item_create	(GType type,
					 const gchar *first_arg_name, ...);

void	dia_canvas_item_request_update	(DiaCanvasItem *item);

void	dia_canvas_item_update_now	(DiaCanvasItem *item);

void	dia_canvas_item_update_child	(DiaCanvasItem *item,
					 DiaCanvasItem *child,
					 gdouble affine[6]);

void	dia_canvas_item_affine_w2i	(DiaCanvasItem *item,
					 gdouble affine[6]);
void	dia_canvas_item_affine_i2w	(DiaCanvasItem *item,
					 gdouble affine[6]);

void	dia_canvas_item_affine_point_w2i (DiaCanvasItem *item,
					  gdouble *x, gdouble *y);
void	dia_canvas_item_affine_point_i2w (DiaCanvasItem *item,
					  gdouble *x, gdouble *y);

void	dia_canvas_item_update_handles_i2w (DiaCanvasItem *item);

void	dia_canvas_item_update_handles_w2i (DiaCanvasItem *item);

gboolean dia_canvas_item_connect	(DiaCanvasItem *item,
					 DiaHandle *handle);
gboolean dia_canvas_item_disconnect	(DiaCanvasItem *item,
					 DiaHandle *handle);

gboolean dia_canvas_item_disconnect_handles (DiaCanvasItem *item);

/* Change the object's state (the state of the active canvas view item): */
void	dia_canvas_item_select		(DiaCanvasItem *item);
void	dia_canvas_item_unselect	(DiaCanvasItem *item);
void	dia_canvas_item_focus		(DiaCanvasItem *item);
void	dia_canvas_item_unfocus		(DiaCanvasItem *item);
void	dia_canvas_item_grab		(DiaCanvasItem *item);
void	dia_canvas_item_ungrab		(DiaCanvasItem *item);

void	dia_canvas_item_visible		(DiaCanvasItem *item);
void	dia_canvas_item_invisible	(DiaCanvasItem *item);

/* Transforming the item: */
void	dia_canvas_item_identity	(DiaCanvasItem *item);

void	dia_canvas_item_scale		(DiaCanvasItem *item,
					 gdouble sx, gdouble sy);

void	dia_canvas_item_rotate		(DiaCanvasItem *item, gdouble degrees);

void	dia_canvas_item_shear_x		(DiaCanvasItem *item,
					 gdouble dx, gdouble dy);
void	dia_canvas_item_shear_y		(DiaCanvasItem *item,
					 gdouble dx, gdouble dy);

void	dia_canvas_item_move		(DiaCanvasItem *item,
					 gdouble dx, gdouble dy);

void	dia_canvas_item_flip		(DiaCanvasItem *item,
					 gboolean horz, gboolean vert);

/* Use this one in events handlers: */
void	dia_canvas_item_move_interactive (DiaCanvasItem *item,
					  gdouble dx, gdouble dy);

void	dia_canvas_item_expand_bounds	(DiaCanvasItem *item, gdouble d);

void	dia_canvas_item_bb_affine	(DiaCanvasItem* item, gdouble affine[6],
					 gdouble *x1, gdouble *y1,
					 gdouble *x2, gdouble *y2);

/* Iterate over shapes: */
gboolean dia_canvas_item_get_shape_iter (DiaCanvasItem *item,
					 DiaCanvasIter *iter);
gboolean dia_canvas_item_shape_next	(DiaCanvasItem *item,
					 DiaCanvasIter *iter);
DiaShape* dia_canvas_item_shape_value	(DiaCanvasItem *item,
					 DiaCanvasIter *iter);

/* Storing undo information the easy way: */
void	dia_canvas_item_preserve_property (DiaCanvasItem *item,
					   const gchar *property_name);

/**
 * DiaCanvasGroupable:
 *
 * DiaCanvasItems supporting this interface can have child objects in them
 * which will be displayed on the #DiaCanvasView as a seperate object.
 **/
struct _DiaCanvasGroupable
{
	GTypeInterface g_iface;

	/* Signals */
	/**
	 * add:
	 * @group:
	 * @item:
	 *
	 * Add an item to the DiaCanvasGroupable object. This method should
	 * take care of setting item->parent and item->canvas.
	 *
	 * Return value: TRUE if the item is added, FALSE otherwise.
	 **/
	gboolean (* add)	(DiaCanvasItem *group, DiaCanvasItem *item);
	gboolean (* remove)	(DiaCanvasItem *group, DiaCanvasItem *item);
	
	/* Callbacks */
	gboolean (* get_iter)	(DiaCanvasItem *group, DiaCanvasIter *iter);
	gboolean (* next)	(DiaCanvasItem *group, DiaCanvasIter *iter);

	DiaCanvasItem* (* value) (DiaCanvasItem *group, DiaCanvasIter *iter);

	gint	(* length)	(DiaCanvasItem *group);
	gint	(* pos)		(DiaCanvasItem *group,  DiaCanvasItem *item);
};

GType	dia_canvas_groupable_get_type	(void);

gboolean dia_canvas_groupable_add	(DiaCanvasItem *group,
					 DiaCanvasItem *item);
gboolean dia_canvas_groupable_remove	(DiaCanvasItem *group,
					 DiaCanvasItem *item);

/* Those methods can be used for compound objects. */
void	dia_canvas_groupable_add_construction	(DiaCanvasItem *group,
						 DiaCanvasItem *item);
void	dia_canvas_groupable_remove_destruction	(DiaCanvasItem *group,
						 DiaCanvasItem *item);

/* Z-order is adjusted by dia_canvas_item_raise/lower(). */

gboolean dia_canvas_groupable_get_iter	(DiaCanvasItem *group,
					 DiaCanvasIter *iter);
gboolean dia_canvas_groupable_next	(DiaCanvasItem *group,
					 DiaCanvasIter *iter);
DiaCanvasItem*	dia_canvas_groupable_value	(DiaCanvasItem *group,
						 DiaCanvasIter *iter);

gint	dia_canvas_groupable_length	(DiaCanvasItem *group);
gint	dia_canvas_groupable_pos	(DiaCanvasItem *group,
					 DiaCanvasItem *item);

/**
 * DiaCanvasGroup
 *
 * DiaCanvasGroup is a special kind of #DiaCanvasItem that can hold
 * other canvas items. It implements the #DiaCanvasGroupable interface.
 **/
struct _DiaCanvasGroup
{
	DiaCanvasItem item;

	GList *children;
};

struct _DiaCanvasGroupClass
{
	DiaCanvasItemClass parent_class;
};


GType	dia_canvas_group_get_type	(void);

DiaCanvasItem* dia_canvas_group_create_item (DiaCanvasGroup *group,
					     GType type,
					     const gchar* first_arg_name, ...);

/* Raise or lower the item wrt its other items. */
void	dia_canvas_group_raise_item	(DiaCanvasGroup *group,
					 DiaCanvasItem *item, gint pos);
void	dia_canvas_group_lower_item	(DiaCanvasGroup *group,
					 DiaCanvasItem *item, gint pos);


typedef gint (* DiaCanvasItemForeachFunc) (DiaCanvasItem *item,
					   gpointer data);

/* If TRUE is returned, the children are not evaluated. */
gint	dia_canvas_group_foreach	(DiaCanvasItem *item,
					 DiaCanvasItemForeachFunc func,
					 gpointer data);


G_END_DECLS

#endif /* __DIA_CANVAS_H__ */
